#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

struct rgb {
  uint8_t r;
  uint8_t g;
  uint8_t b;
};

struct ppm {
  size_t width;
  size_t height;
  struct rgb data[];
};

static struct ppm* ppm_create(size_t width, size_t height)
{
  struct ppm *p = malloc(sizeof(*p) + width * height * 3);
  p->width = width;
  p->height = height;
  return p;
}

static struct ppm* ppm_read(struct ppm *p)
{
  size_t width, height;
  if (scanf("P6%*[^'\n']%zu%zu%*d%*c", &width, &height) < 2) {
    free(p);
    return 0;
  }
  if (!p || p->width != width || p->height != height) {
    free(p);
    p = ppm_create(width, height);
  }
  fread(p->data, width * height, 3, stdin);
  return p;
}

int main(void)
{
  struct ppm *image = 0;
  image = ppm_read(image);
  if (!image)
    return -1;

  uint8_t *data = malloc(image->width * image->height);
  memset(data, 0, image->width * image->height);

  size_t index = 0;
  for (size_t y = 0; y < image->height; y++) {
    for (size_t x = 0; x < image->width; x++) {
      struct rgb pixel = image->data[y * image->width + x];
      if (pixel.r == 0x00 && pixel.g == 0x00 && pixel.b == 0x00)
	data[index] = 0;
      else if (pixel.r == 0xFF && pixel.g == 0x00 && pixel.b == 0x00)
	data[index] = 1;
      else if (pixel.r == 0x00 && pixel.g == 0xFF && pixel.b == 0x00)
	data[index] = 2;
      else if (pixel.r == 0x00 && pixel.g == 0x00 && pixel.b == 0xFF)
	data[index] = 3;
      index++;
    }
  }
  fprintf(stderr, "%lu, %lu,\n", image->width, image->height);
  fflush(stderr);

  for (size_t i = 0; i < image->width * image->height; i++)
    fprintf(stdout, "%s0x%02x,", (i % 16) ? " " : "\n  "+1*!i, data[i]);
  fprintf(stdout, "\n");
  fflush(stdout);

  free(data);
  return 0;
}
